import React, { createContext, useContext } from 'react';
import { useGovVoteForm } from '@onomy/react-hub';

import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';

function useProposalVoteState({ proposalId }: { proposalId: string }) {
  const state = useGovVoteForm({ proposalId });
  return state;
}

type ProposalVoteProps = Parameters<typeof useProposalVoteState>[0];

export type ProposalVoteContextType = ReturnType<typeof useProposalVoteState>;

export const ProposalVoteContext = createContext<ProposalVoteContextType>(null as any);

export function useProposalVote() {
  const state = useContext(ProposalVoteContext);
  if (!state) throw new Error('Missing ProposalVoteContext');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading vote form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ProposalVoteProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  proposalId: string;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
} & ProposalVoteProps) {
  const state = useProposalVoteState(props);

  if (state.loadingStatus.pending || !state.proposal) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);

  return <ProposalVoteContext.Provider value={state}>{children}</ProposalVoteContext.Provider>;
}
