// eslint-disable
import BigNumber from 'bignumber.js';

export const PROPOSAL_STATUS = {
  PROPOSAL_STATUS_UNSPECIFIED: 'Unspecified',
  PROPOSAL_STATUS_DEPOSIT_PERIOD: 'Deposit Period',
  PROPOSAL_STATUS_VOTING_PERIOD: 'Voting Period',
  PROPOSAL_STATUS_PASSED: 'Passed',
  PROPOSAL_STATUS_REJECTED: 'Rejected',
  PROPOSAL_STATUS_FAILED: 'Failed',
  UNRECOGNIZED: 'Unrecognized',
};

export const PROPOSAL_TYPE = {
  ParameterChangeProposal: {
    name: 'Parameter Change',
    // icon: iconParameter,
  },
  FundAccountProposal: {
    name: 'Fund Account',
    // icon: iconFunding,
  },
  SoftwareUpgradeProposal: {
    name: 'Software Upgrade',
    // icon: iconDefault,
  },
};

export function isUndefined(index: any): index is boolean {
  return typeof index === 'undefined';
}

export function getProposalType(index?: string) {
  const undefinedType = {
    name: 'Unknown',
    // icon: iconDefault,
  };

  return isUndefined(index)
    ? undefinedType
    : // @ts-ignore
      PROPOSAL_TYPE[index!] ?? {
        name: index?.split('/').pop(),
        // icon: iconDefault,
      };
}

export function getProposalStatus(index?: string) {
  // @ts-ignore
  return isUndefined(index) ? 'Unknown' : PROPOSAL_STATUS[index!];
}

export function renderRemainingDays(endDate: Date | null): string {
  if (!endDate) return 'No end date specified';

  const timeDifference = (Date.parse(endDate.toString()) - Date.now()) / 1000 / 60 / 60 / 24;

  if (timeDifference < 0) return 'Voting Ended';
  if (timeDifference < 1) return 'Less than 1 day left';
  if (timeDifference < 2) return '1 day left';

  return `${Math.floor(timeDifference)} days left`;
}

export function calculateVotesChartData(proposal: any) {
  return proposal?.results?.total
    ? [
        new BigNumber(proposal.results?.yes ?? 0)
          .dividedBy(proposal.results.total)
          .times(100)
          .toNumber(),
        new BigNumber(proposal.results?.no ?? 0)
          .plus(proposal.results?.noWithVeto ?? 0)
          .dividedBy(proposal.results.total)
          .times(100)
          .toNumber(),
        new BigNumber(proposal.results?.abstain ?? 0)
          .dividedBy(proposal.results.total)
          .times(100)
          .toNumber(),
      ].map(x => {
        if (Number.isNaN(x)) return 0;
        return x;
      })
    : [0, 0, 0];
}
