import React, { createContext, useContext } from 'react';
import { useMoveDelegationForm } from '@onomy/react-hub';

import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';

function useMoveDelegationState(props: Parameters<typeof useMoveDelegationForm>[0]) {
  const state = useMoveDelegationForm(props);
  return state;
}

type MoveDelegationProps = Parameters<typeof useMoveDelegationState>[0];

export type MoveDelegationContextType = ReturnType<typeof useMoveDelegationState>;

export const MoveDelegationContext = createContext<MoveDelegationContextType>(null as any);

export function useMoveDelegation() {
  const state = useContext(MoveDelegationContext);
  if (!state) throw new Error('Missing MoveDelegationContext');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading move delegation form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function MoveDelegationProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  proposalId: string;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
} & MoveDelegationProps) {
  const state = useMoveDelegationState(props);

  if (state.loadingStatus.pending || !state.srcDelegation) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);

  return <MoveDelegationContext.Provider value={state}>{children}</MoveDelegationContext.Provider>;
}
