import React, { useMemo } from 'react';
import { OnomyHubProvider, RequireOnomyHub } from '@onomy/react-hub';
import { WalletBackend } from '@onomy/wallet';
import { OnomyAppProvider } from './OnomyApp';
import {
  OnomyEcosystemConfigName,
  setOnomyEcosystemConfig,
  OnomyEcosystemConfig,
} from '../../OnomyEcosystemConfigs';

export function OnomyCoreProvider({
  children,
  createBackendFn,
  config = 'DEVNET',
  overrides,
  renderLoading,
  renderError,
}: {
  children: JSX.Element | JSX.Element[];
  createBackendFn: () => WalletBackend;
  config?: OnomyEcosystemConfigName;
  overrides?: Partial<OnomyEcosystemConfig>;
  renderLoading?: Parameters<typeof OnomyAppProvider>[0]['renderLoading'];
  renderError?: Parameters<typeof OnomyAppProvider>[0]['renderError'];
}) {
  const ecosystem = useMemo(() => {
    return setOnomyEcosystemConfig(config, overrides);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  // eslint-disable-next-line react-hooks/exhaustive-deps
  const backend = useMemo(createBackendFn, []);
  return (
    <OnomyHubProvider
      backend={backend}
      onomyChainInfo={ecosystem.onomyChainInfo}
      onexChainInfo={ecosystem.onexChainInfo}
      ethWsUrl={ecosystem.ethWsUrl}
      nomContractAddress={ecosystem.bNomContractAddress}
      bondContractAddress={ecosystem.bondContractAddress}
      gravityContractAddress={ecosystem.gravityContractAddress}
      wEthUsdtUniswapPoolAddress={ecosystem.wEthUsdUniswapPoolAddress}
    >
      <RequireOnomyHub>
        <OnomyAppProvider
          ecosystem={ecosystem}
          renderLoading={renderLoading}
          renderError={renderError}
        >
          {children}
        </OnomyAppProvider>
      </RequireOnomyHub>
    </OnomyHubProvider>
  );
}
